@@ -20,24 +20,28 @@
 #ifndef ACTIVITIES_P_H
 #define ACTIVITIES_P_H
 
+// Self
+#include "Activities.h"
+
+// Qt
 #include <QString>
 #include <QTimer>
+#include <QReadWriteLock>
 
-#include <KConfig>
-#include <KConfigGroup>
+// KDE
+#include <kconfig.h>
+#include <kconfiggroup.h>
 
-#include "Activities.h"
 
 class KSMServer;
 
 class QDBusInterface;
-class KJob;
 
-class Activities::Private: public QObject {
+class Activities::Private : public QObject {
     Q_OBJECT
 
 public:
-    Private(Activities * parent);
+    Private(Activities *parent);
     ~Private();
 
     // Loads the last activity
@@ -49,29 +53,61 @@ public:
     void ensureCurrentActivityIsRunning();
 
 public Q_SLOTS:
-    bool setCurrentActivity(const QString & activity);
+    bool setCurrentActivity(const QString &activity);
 
 public:
-    void setActivityState(const QString & activity, Activities::State state);
-    QHash < QString, Activities::State > activities;
-
-    // Current activity
-    QString currentActivity;
+    void setActivityState(const QString &activity, Activities::State state);
 
     // Configuration
+    class ConfigurationChecker {
+    public:
+        ConfigurationChecker();
+    } configChecker;
     QTimer configSyncTimer;
     KConfig config;
 
     // Interface to the session management
-    KSMServer * ksmserver;
+    KSMServer *ksmserver;
+
+    QHash<QString, Activities::State> activities;
+    QReadWriteLock activitiesLock;
+    QString currentActivity;
 
 public:
-    KConfigGroup activitiesConfig();
-    KConfigGroup activityIconsConfig();
-    KConfigGroup mainConfig();
-    QString activityName(const QString & activity);
-    QString activityIcon(const QString & activity);
+    inline KConfigGroup activityNameConfig()
+    {
+        return KConfigGroup(&config, "activities");
+    }
 
+    inline KConfigGroup activityDescriptionConfig()
+    {
+        return KConfigGroup(&config, "activities-descriptions");
+    }
+
+    inline KConfigGroup activityIconConfig()
+    {
+        return KConfigGroup(&config, "activities-icons");
+    }
+
+    inline KConfigGroup mainConfig()
+    {
+        return KConfigGroup(&config, "main");
+    }
+
+    inline QString activityName(const QString &activity)
+    {
+        return activityNameConfig().readEntry(activity, QString());
+    }
+
+    inline QString activityDescription(const QString &activity)
+    {
+        return activityDescriptionConfig().readEntry(activity, QString());
+    }
+
+    inline QString activityIcon(const QString &activity)
+    {
+        return activityIconConfig().readEntry(activity, QString());
+    }
 
 public Q_SLOTS:
     // Schedules config syncing to be done after
@@ -83,14 +119,12 @@ public Q_SLOTS:
     // Immediately syncs the configuration file
     void configSync();
 
-    void removeActivity(const QString & activity);
-    void activitySessionStateChanged(const QString & activity, int state);
-
-    void emitCurrentActivityChanged(const QString & activity);
+    QString addActivity(const QString &name);
+    void removeActivity(const QString &activity);
+    void activitySessionStateChanged(const QString &activity, int state);
 
 private:
-    Activities * const q;
+    Activities *const q;
 };
 
 #endif // ACTIVITIES_P_H
-
